`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2024/06/24 
// Design Name: 
// Module Name: spi_master
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies:  V0.2  Corresponding to the code of June 19, the spi slave side of the three-state port output
// 				  Fixed a bug where high resistance z was sampled during the first falling edge sampling		
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module spi_master
 #(parameter SPI_MODE = 3)
    (
    // control signal
    input clk,
    input reset,
    
    //// TX(MOSI) signal
    input         Rx_ready,    // ÐèÒª¶ÁÐ´spiÊ±ÖÃ1
    input [7 : 0] axi_byte,
    input         axi_valid,
    
    output        ss,      // Æ¬Ñ¡ÐÅºÅ
    output        spi_master_bit,
    output        spi_clk,
    
    // RX(MISO) signal
//    output [7 : 0] RX_byte,
//    output         RX_valid,
    input          spi_slave_bit,
    output [7 : 0] spi_master_byte,
    output         spi_master_valid
    
    // SPI Interface   
    
    );
wire w_CPOL;     // Clock polarity
wire w_CPHA;     // Clock phase

assign w_CPOL  = (SPI_MODE == 2) | (SPI_MODE == 3);
assign w_CPHA  = (SPI_MODE == 1) | (SPI_MODE == 3);    

reg Rx_ready_r1;
reg Rx_ready_r2;
always@(posedge clk)
begin
    if(reset)
    begin
        Rx_ready_r1 <= 1'b0;
        Rx_ready_r2 <= 1'b0;
    end
    else
    begin
        Rx_ready_r1 <= Rx_ready;
        Rx_ready_r2 <= Rx_ready_r1;
    end
end

reg spi_clk_r;
reg spi_clk_rr; 
always@(posedge clk)
begin
    if(reset)
    begin
        spi_clk_r <= 1'b1;
        //spi_clk_rr <= 1'b1;
    end
    else
    begin 
        if(Rx_ready)
        begin
            spi_clk_r <= spi_clk_r + 1'b1;
        end
        
        //spi_clk_rr <= spi_clk_r; // 2·ÖÆµ
    end
end
always@(posedge clk)
begin
    if(reset)
    begin
        spi_clk_rr <= 1'b1;
    end
    else
    begin 
        if(spi_clk_r == 1'b0)
        spi_clk_rr <= ~spi_clk_rr; //4·ÖÆµ
    end
end
reg ss_r;
reg ss_rr;
wire rx_edge;
always@(negedge clk)    // only ss need to read negedge of clk
begin
    if(reset)
    begin
        ss_r <= 1'b1;
        ss_rr <= ss_r;
       // rx_edge_r <= 1'b0;
    end
    else
    begin 
        ss_r <= ~Rx_ready;
        ss_rr <= ss_r;
       // rx_edge_r <= rx_edge;
    end
end

wire tx_edge;
assign rx_edge = (spi_clk_r) & (~spi_clk_rr);  // mode 3
assign tx_edge = (~spi_clk_r) & (spi_clk_rr);

reg rx_edge_r;
reg rx_edge_r2;
reg  rx_edge_r3;
always@(posedge clk)
begin
    if(reset)
    begin
        rx_edge_r <= 1'b0;
        rx_edge_r2 <= 1'b0;
        rx_edge_r3 <= 1'b0;
    end
    else
    begin 
        rx_edge_r <= rx_edge;
        rx_edge_r2 <= rx_edge_r;
        rx_edge_r3 <= rx_edge_r2;
    end
end

reg spi_master_bit_r;
//reg spi_master_bit_rr;
reg [7 : 0] spi_master_byte_r;
reg [2 : 0] bit_cnt1; // 0~7
reg [2 : 0] bit_cnt2; // 0~7
always@(posedge clk)
begin
//    if(reset || (~Rx_ready))
//    begin
//        spi_master_bit_r <= 1'b0;
//        spi_master_bit_rr <= 1'b0;
//        spi_master_byte_r <= 8'b0;
//        bit_cnt1 <= 3'd7;
//        bit_cnt2 <= 3'd7;
//    end
    if(reset)
    begin
        spi_master_bit_r <= 1'b0;
        //spi_master_bit_rr <= 1'b0;
        spi_master_byte_r <= 8'b0;
        bit_cnt1 <= 3'd7;
        bit_cnt2 <= 3'd7;
    end
    else if (~Rx_ready_r1)
    begin
        bit_cnt1 <= 3'd7;
        bit_cnt2 <= 3'd7;
    end
    else
    begin       
        if(tx_edge)
        begin
            spi_master_bit_r <= axi_byte[bit_cnt1];  // spi_master_bit_r Ó¦¸Ã°´spi clk ¶ø²»ÊÇ clk ±ä»¯
            bit_cnt1 <= bit_cnt1 - 1'b1;
            //spi_master_bit_rr <= spi_master_bit_r;  
        end
        else if(rx_edge_r2)
        begin
            spi_master_byte_r[bit_cnt2] <= spi_slave_bit;
            bit_cnt2 <= bit_cnt2 - 1'b1;
        end    
    end
end

assign spi_clk = spi_clk_rr;
assign ss = ss_rr;
assign spi_master_bit = spi_master_bit_r;



assign spi_master_byte = spi_master_byte_r;
assign spi_master_valid = ((bit_cnt2 == 3'd7) && (rx_edge_r3 == 1'b1)) ? 1'b1 : 1'b0;


/*
reg spi_master_valid_reg;
always@(posedge clk)
begin
    if(reset)begin
        spi_master_valid_reg <= 1'b0;
    end else if(spi_master_valid & Rx_ready)begin
        spi_master_valid_reg <= 1'b1;
    end else if(~Rx_ready)begin
        spi_master_valid_reg <= 1'b0;
    end
end
assign miso_valid = spi_master_valid & spi_master_valid_reg;
*/
endmodule

