//+FHDR--------------------------------------------------------------------------------------------------------
//  Company: 
//-----------------------------------------------------------------------------------------------------------------
//  File Name             :    spi_to_sram_v1.2.v
//  Department            :    
//  Author                :    ZYZ
//  Author's Tel          :     
//-----------------------------------------------------------------------------------------------------------------
//  Relese History
//  Version     Date            Author          Description
//  0.1         2024-05-30       ZYZ
//-----------------------------------------------------------------------------------------------------------------
//  Keywords            :      (1) connect rx_sram , tx_sram , spi-master ,digital_top successfully,the
//                         verilog files of the digital_top is marked 2024.5.17   
//			       (2) when reading data,tx_sram still need to complete the data 
//
//-----------------------------------------------------------------------------------------------------------------
//  Parameter
//
//-----------------------------------------------------------------------------------------------------------------
//  Purpose                 :
//                      
//-----------------------------------------------------------------------------------------------------------------
//  Target Device:        
//  Tool versions:        
//-----------------------------------------------------------------------------------------------------------------
//  Reuse Issues
//  Reset Strategy: 
//  Clock Domains: 
//  Critical Timing:
//  Asynchronous I/F:
//  Synthesizable (y/n): 
//  Other:
//-FHDR--------------------------------------------------------------------------------------------------------

`include "../../rtl/define/chip_define.v"

module sram_z_top(
	 input clk
	,input por_rstn
	
	,input          async_rstn        // hardware Reset, active low           
	//sync
	,input          sync_in           // Chip synchronization signal input, high pulse valid
	,output         sync_out          // Chip synchronization signal output, high pulse valid
	
	,input   [1 :0] ch0_feedback      // Ch0 Feedback signals from the readout chip
    `ifdef CHANNEL_IS_FOUR
	,input   [1 :0] ch1_feedback      // Ch1 Feedback signals from the readout chip
	,input   [1 :0] ch2_feedback      // Ch2 Feedback signals from the readout chip
	,input   [1 :0] ch3_feedback      // Ch3 Feedback signals from the readout chip
    `endif
	,input   [4 :0] cfgid             // During power-on initialization, the IO configuration 
                                    // values are read as the chip ID number
    //irq
	,output 		irq
	//tx_sram port
	,input [31	:0]	din
	,input [15	:0]	data_length
	,output 		data_done_all
	,output 		data_out_per
        ,input                  ch0_dac_Cal_end 
        ,input                  ch0_dc_Cal_end   
	//digital_top port
	,output 		oen
	//rx_sram port
	,input			data_rden_rx
	,output	[31:0]		Rdata_PC
  //------------------------------Ch0 DSP data out----------------------------------------------------
  ,output [15 :0] ch0_z_dsp_dout0
  ,output [15 :0] ch0_z_dsp_dout1
  ,output [15 :0] ch0_z_dsp_dout2
  ,output [15 :0] ch0_z_dsp_dout3 
  ,output         ch0_z_DEM_MSB_vld 
);

//tx_sram signal
wire [31:0]	Nlen;
wire [4 :0]	chip_ID;
wire [24:0]	address;
wire [31:0]	data_out_rx;
wire 		wr_en;
wire		rd_en;
wire		RREADY;
wire		WREADY;
//axi-spi signal
wire 		spi_master_bit;
wire		spi_clk;
wire		ss;
wire		spi_slave_bit;
//rx_sram signal
wire [31:0]		RDATA;
wire			RVALID;

wire  [14 :0]  ch0_z_DEM_MSB_OUT0; 
wire  [14 :0]  ch0_z_DEM_MSB_OUT1;
wire  [14 :0]  ch0_z_DEM_MSB_OUT2;
wire  [14 :0]  ch0_z_DEM_MSB_OUT3; 
wire  [6  :0]  ch0_z_DEM_ISB_OUT0;
wire  [6  :0]  ch0_z_DEM_ISB_OUT1;
wire  [6  :0]  ch0_z_DEM_ISB_OUT2;
wire  [6  :0]  ch0_z_DEM_ISB_OUT3;
wire  [8  :0]  ch0_z_DEM_LSB_OUT0;
wire  [8  :0]  ch0_z_DEM_LSB_OUT1;
wire  [8  :0]  ch0_z_DEM_LSB_OUT2;
wire  [8  :0]  ch0_z_DEM_LSB_OUT3;

tx_sram inst_tx_sram(
   	.clk(clk),
   	.rstn(por_rstn),
	//input
   	.dina(din),
	.data_length (data_length),

	.WREADY			(WREADY),
	.RREADY			(RREADY),
	//output
	.Nlen 			(Nlen),
	.chip_ID		(chip_ID),
	.address 		(address),
    	.data_out 		(data_out_rx),
	.wr_en 			(wr_en),
	.rd_en 			(rd_en),
	
	.data_done_all  (data_done_all),
	.data_out_per 	(data_out_per)
);


AxiSpi inst_AxiSpi(
    .clk         (clk  		),
    .reset       (!por_rstn	),
    .WR		 (wr_en		),
    .RD          (rd_en		),          
   //input
    .WADDR       ({7'b0,address}),      
    .RADDR  	 ({7'b0,address}),    
    .DIN 	 (data_out_rx   ),		 	
    .WVALID      (			),     
    .cmd_s       (			),      
    .chirpID 	 (chip_ID	),
    .Nlen 	 (Nlen		),		
    //output
    .WREADY      (WREADY	),      
    .RREADY      (RREADY	),    
    // read data from spi slave, need to send to axi
    .RVALID      (RVALID	),      	
    .RDATA 	 (RDATA 	),   		
    
    // interface to spi slave
    .spi_slave_bit    (spi_slave_bit ),	//miso
    
    .ss               (ss  		     ),				//ss
    .spi_clk          (spi_clk		 ),		//2 or 4
    .spi_master_bit   (spi_master_bit)	//mosi
    );


rx_sram inst_rx_sram(
    .clk 			(clk),
    .rstn 			(por_rstn),
	
    .din        		(RDATA),
    .din_vld		        (RVALID),
	
    .data_rden_rx 	        (data_rden_rx),
    .Rdata_PC	  	        (Rdata_PC)
);
	
	
z_chip_top inst_chip_top (   
  //+++++++++++++++++++++++++++++++++++++++++++++//
  //                 PAD Strat                   //
  //+++++++++++++++++++++++++++++++++++++++++++++//
   .PI_async_rstn       (async_rstn  )// hardware Reset, active low           
  //sync
  ,.PI_sync_in          (sync_in     ) // Chip synchronization signal input, high pulse valid
  ,.PO_sync_out         (sync_out    ) // Chip synchronization signal output, high pulse valid
  //Feedback signal
  ,.PI_ch0_feedback     (ch0_feedback)// Ch0 Feedback signals from the readout chip
  `ifdef CHANNEL_IS_FOUR
  ,.PI_ch1_feedback     (ch1_feedback) // Ch1 Feedback signals from the readout chip
  ,.PI_ch2_feedback     (ch2_feedback) // Ch2 Feedback signals from the readout chip
  ,.PI_ch3_feedback     (ch3_feedback) // Ch3 Feedback signals from the readout chip
  `endif
  //config chip id
  ,.PI_cfgid            (cfgid)// During power-on initialization, the IO configuration 
                                       // values are read as the chip ID number
  //spi port                        
  ,.PI_sclk             (spi_clk       ) // Spi Clock
  ,.PI_csn              (ss	       ) // Spi Chip Select active low
  ,.PI_mosi             (spi_master_bit) // Spi Mosi
  ,.PO_miso             (spi_slave_bit ) // Spi Miso
  //irq
  ,.PO_irq              (irq           )// Interrupt signal in the chip, high level active
  //Attenuator Control Bit\u200c
  ,.PO_ch0_att          (              )     // CH0 Attenuator Control Bit
  `ifdef CHANNEL_IS_FOUR
  ,.PO_ch1_att          (              )     // CH1 Attenuator Control Bit
  ,.PO_ch2_att          (              )     // CH2 Attenuator Control Bit
  ,.PO_ch3_att          (              )     // CH3 Attenuator Control Bit
  `endif
  //+++++++++++++++++++++++++++++++++++++++++++++//
  //                  PAD End                    //                    
  //+++++++++++++++++++++++++++++++++++++++++++++//
  //+++++++++++++++++++++++++++++++++++++++++++++//
  //                  PIN Strat                  //                    
  //+++++++++++++++++++++++++++++++++++++++++++++//
    //-------------------------clcok pin from pll-------------------------------------------------
  ,.clk                 (clk           )// System Main Clock
  //-------------------------Power on reset pin from por---------------------------------------- 
  ,.por_rstn            (por_rstn      )// Power on reset, active low 
  //------------------------------digital IO----------------------------------------------------
  //------------------------------PLL cfg pin----------------------------------------------------
  ,.ref_sel             (              )// Clock source selection for a frequency divider; 
                                    // 1'b0:External clock source
            // 1'b1:internal phase-locked loop clock source
  ,.ref_en              (              )// Input reference clock enable
                                    // 1'b0:enable,1'b1:disable
  ,.ref_s2d_en          (              )// Referenced clock differential to single-ended conversion enable
                                    // 1'b0:enable,1'b1:disable
  ,.p_cnt               (              ) // P counter
  ,.pfd_delay           (              ) // PFD Dead Zone
  ,.pfd_dff_Set         (              ) // Setting the PFD register,active high
  ,.pfd_dff_4and        (              ) // PFD output polarity
  ,.spd_div             (              ) // SPD Frequency Divider
  ,.spd_pulse_width     (              ) // Pulse Width of SPD
  ,.spd_pulse_sw        (              ) // Pulse sw of SPD
  ,.cpc_sel             (              ) // current source selection
  ,.swcp_i              (              ) // PTAT current switch
  ,.sw_ptat_r           (              ) // PTAT current adjustment
  //,output [1  :0] sw_fll_cpi        // Phase-locked loop charge pump current
  //,output         sw_fll_delay      // PLL Dead Zone
  ,.pfd_sel             (              ) // PFD Loop selection
  ,.spd_sel             (              ) // SPD Loop selection
  ,.dtd_en              (              ) // 
  //,output         fll_sel           // FLL Loop selection
  ,.vco_tc              (              ) // VCO temperature compensation
  ,.vco_tcr             (              ) // VCO temperature compensation resistor
  ,.vco_gain_adj        (              ) // VCO gain adjustment
  ,.vco_gain_adj_r      (              ) // VCO gain adjustment resistor
  ,.vco_cur_adj         (              ) // VCO current adjustment
  ,.vco_buff_en         (              ) // VCO buff enable,active high
  ,.vco_en              (              ) // VCO enable,active high
  ,.vco_fb_adj           (              ) // VCO frequency band adjustment
//  ,.pll_dpwr_adj        (            ) // PLL frequency division output power adjustment
// ,output [7  :0] vco_fb_adj        // VCO frequency band adjustment
  ,.tc_sel		(              )    	 // Temperature compensation options
  ,.pata_res_en     	(              )	 // PATAMOS resistance enable
  ,.pata_s_dc_sel   	(              )	 // PATA slope DC selection
  ,.pata_res_vdc300 	(              )	 // PATAMOS resistance bias
  ,.pata_res_vdc500 	(              )	 // PATAMOS resistance bias
  ,.pata_res_vdc800 	(              )	 // PATAMOS resistance bias
  ,.sw_var_temp_en  	(              )	 // Temperature-compensated automatic switch
  ,.sw_var_temp     	(              )	 // Temperature-compensated manual switch
  
  ,.afc_en              (              ) // AFC enable
  ,.afc_shutdown        (              ) // AFC module shutdown signal
  ,.afc_reset           (              ) // AFC reset
  //,output [0  :0] afc_det_speed    // AFC detection speed 
  //,output [0  :0] flag_out_sel     // Read and choose the signs
  ,.afc_clk_sel         (              ) 	 // AFC clock frequency choose
  ,.afc_pres            (              )   // AFC comparator accuracy adjustment
  
  ,.afc_fb_cnt          (              )	// AFC frequency band adjustment function counter
  ,.afc_fb_target       (              )	// AFC frequency band adjustment function target number of cycles

  ,.afc_ld_cnt          (              ) // Adjust the counting time of the AFC lock detection                                    // feature counter         
  ,.afc_ld_target       (              )// AFC Lock Detection Function Target Cycle Count

  ,.div_rstn_sel        (              ) // div rstn select, 1'b0: ext clear, 1'b1:inter pll lock
  ,.test_clk_sel        (              ) // test clk select, 4'b0001:DIV1 clk, 4'b0010:DIV2 clk, 4'b0100:DIV4 clk, 4'b1000:DIV8 clk
  ,.test_clk_oen        (              ) // test clk output enable, 1'b0:disenable, 1'b1:enable
  ,.dig_clk_sel         (              ) // digital main clk select, one hot code,bit[0]-->0 degree phase,bit[1]-->45 degree phase.....bit[7]-->315degree  phae 
  ,.clkrx_pdn           (              ) // CLock Rx Power Down
  ,.sync_clr            (              ) // PLL div sync clr,low active
  ,.pll_rstn            (              ) // PLL reset,active low
  ,.pll_lock            (              ) // PLL LOCK
  ,.afc_end_flag        (              )
  ,.clk_resv            (              )
  //------------------------------Ch0 DAC cfg pin----------------------------------------------------     
  ,.ch0_dac_addr        ()  
  ,.ch0_dac_dw          () 
  ,.ch0_dac_ref         ()
  ,.ch0_dac_Prbs_rst0   ()
  ,.ch0_dac_Prbs_set0   ()
  ,.ch0_dac_Prbs_rst1   ()  
  ,.ch0_dac_Prbs_set1   ()  
  ,.ch0_dac_Cal_sig     () 
  ,.ch0_dac_Cal_rstn    ()
  ,.ch0_dac_Cal_div_rstn()  
  ,.ch0_dac_Cal_end     (ch0_dac_Cal_end) 
  ,.ch0_dac_Ctrlp       () 
  ,.ch0_dac_Ctrln       () 
  //------------------------------Ch0 DC Bias cfg pin----------------------------------------------------  
  ,.ch0_dc_addr         () 
  ,.ch0_dc_dw           () 
  ,.ch0_dc_ref          () 
  ,.ch0_dc_Cal_sig      ()  
  ,.ch0_dc_Cal_rstn     () 
  ,.ch0_dc_Cal_div_rstn () 
  ,.ch0_dc_Cal_end      (ch0_dc_Cal_end)
  ,.ch0_dc_clkout       () 
  `ifdef CHANNEL_IS_FOUR  
  //------------------------------Ch1 DAC cfg pin----------------------------------------------------
  ,.ch1_dac_addr        () 
  ,.ch1_dac_dw          () 
  ,.ch1_dac_ref         () 
  ,.ch1_dac_Prbs_rst0   () 
  ,.ch1_dac_Prbs_set0   ()  
  ,.ch1_dac_Prbs_rst1   () 
  ,.ch1_dac_Prbs_set1   () 
  ,.ch1_dac_Cal_sig     () 
  ,.ch1_dac_Cal_rstn    ()
  ,.ch1_dac_Cal_div_rstn()     
  ,.ch1_dac_Cal_end     ()  
  ,.ch1_dac_Ctrlp       () 
  ,.ch1_dac_Ctrln       ()
  //------------------------------Ch1 DC Bias cfg pin----------------------------------------------------  
  ,.ch1_dc_addr         ()  
  ,.ch1_dc_dw           ()  
  ,.ch1_dc_ref          () 
  ,.ch1_dc_Cal_sig      () 
  ,.ch1_dc_Cal_rstn     ()
  ,.ch1_dc_Cal_div_rstn () 
  ,.ch1_dc_Cal_end      () 
  ,.ch1_dc_clkout       () 
  //------------------------------Ch2 DAC cfg pin----------------------------------------------------
  ,.ch2_dac_addr        () 
  ,.ch2_dac_dw          () 
  ,.ch2_dac_ref         () 
  ,.ch2_dac_Prbs_rst0   () 
  ,.ch2_dac_Prbs_set0   ()  
  ,.ch2_dac_Prbs_rst1   () 
  ,.ch2_dac_Prbs_set1   () 
  ,.ch2_dac_Cal_sig     () 
  ,.ch2_dac_Cal_rstn    ()
  ,.ch2_dac_Cal_div_rstn()     
  ,.ch2_dac_Cal_end     () 
  ,.ch2_dac_Ctrlp       () 
  ,.ch2_dac_Ctrln       ()  
  //------------------------------Ch2 DC Bias cfg pin----------------------------------------------------  
  ,.ch2_dc_addr         ()        
  ,.ch2_dc_dw           ()     
  ,.ch2_dc_ref          ()
  ,.ch2_dc_Cal_sig      () 
  ,.ch2_dc_Cal_rstn     ()
  ,.ch2_dc_Cal_div_rstn () 
  ,.ch2_dc_Cal_end      ()
  ,.ch2_dc_clkout       () 
  //------------------------------Ch3 DAC cfg pin----------------------------------------------------
  ,.ch3_dac_addr        () 
  ,.ch3_dac_dw          () 
  ,.ch3_dac_ref         () 
  ,.ch3_dac_Prbs_rst0   () 
  ,.ch3_dac_Prbs_set0   ()  
  ,.ch3_dac_Prbs_rst1   () 
  ,.ch3_dac_Prbs_set1   () 
  ,.ch3_dac_Cal_sig     () 
  ,.ch3_dac_Cal_rstn    () 
  ,.ch3_dac_Cal_div_rstn()     
  ,.ch3_dac_Cal_end     () 
  ,.ch3_dac_Ctrlp       () 
  ,.ch3_dac_Ctrln       ()  
//------------------------------Ch3 DC Bias cfg pin----------------------------------------------------  
  ,.ch3_dc_addr         ()     
  ,.ch3_dc_dw           ()     
  ,.ch3_dc_ref          ()
  ,.ch3_dc_Cal_sig      () 
  ,.ch3_dc_Cal_rstn     ()
  ,.ch3_dc_Cal_div_rstn ()  
  ,.ch3_dc_Cal_end      ()
  ,.ch3_dc_clkout       () 
  `endif
  `ifdef CHANNEL_Z_ON
  //------------------------------Ch0 DSP data out----------------------------------------------------
  ,.ch0_z_DEM_MSB_OUT0  (ch0_z_DEM_MSB_OUT0)
  ,.ch0_z_DEM_MSB_OUT1  (ch0_z_DEM_MSB_OUT1) 
  ,.ch0_z_DEM_MSB_OUT2  (ch0_z_DEM_MSB_OUT2)
  ,.ch0_z_DEM_MSB_OUT3  (ch0_z_DEM_MSB_OUT3) 
  ,.ch0_z_DEM_ISB_OUT0  (ch0_z_DEM_ISB_OUT0)
  ,.ch0_z_DEM_ISB_OUT1  (ch0_z_DEM_ISB_OUT1)
  ,.ch0_z_DEM_ISB_OUT2  (ch0_z_DEM_ISB_OUT2)
  ,.ch0_z_DEM_ISB_OUT3  (ch0_z_DEM_ISB_OUT3)
  ,.ch0_z_DEM_LSB_OUT0  (ch0_z_DEM_LSB_OUT0)
  ,.ch0_z_DEM_LSB_OUT1  (ch0_z_DEM_LSB_OUT1)
  ,.ch0_z_DEM_LSB_OUT2  (ch0_z_DEM_LSB_OUT2)
  ,.ch0_z_DEM_LSB_OUT3  (ch0_z_DEM_LSB_OUT3)
  ,.ch0_z_DEM_MSB_vld   (ch0_z_DEM_MSB_vld )
  ,.ch0_dc_bias_o       ( )
  ,.ch0_dc_bias_latch   ( )
  `endif 
  `ifdef CHANNEL_IS_FOUR
   //------------------------------Ch1 DSP data out----------------------------------------------------
  `ifdef CHANNEL_Z_ON
  ,.ch1_z_DEM_MSB_OUT0  ( ) 
  ,.ch1_z_DEM_MSB_OUT1  ( )
  ,.ch1_z_DEM_MSB_OUT2  ( )
  ,.ch1_z_DEM_MSB_OUT3  ( ) 
  ,.ch1_z_DEM_ISB_OUT0  ( )
  ,.ch1_z_DEM_ISB_OUT1  ( )
  ,.ch1_z_DEM_ISB_OUT2  ( )
  ,.ch1_z_DEM_ISB_OUT3  ( )
  ,.ch1_z_DEM_LSB_OUT0  ( )
  ,.ch1_z_DEM_LSB_OUT1  ( )
  ,.ch1_z_DEM_LSB_OUT2  ( )
  ,.ch1_z_DEM_LSB_OUT3  ( )
  ,.ch1_z_DEM_MSB_vld   ( )
  ,.ch1_dc_bias_o       ( )
  ,.ch1_dc_bias_latch   ( )
  `endif  
    //------------------------------Ch2 DSP data out----------------------------------------------------
  `ifdef CHANNEL_Z_ON
  ,.ch2_z_DEM_MSB_OUT0  ( ) 
  ,.ch2_z_DEM_MSB_OUT1  ( )
  ,.ch2_z_DEM_MSB_OUT2  ( )
  ,.ch2_z_DEM_MSB_OUT3  ( ) 
  ,.ch2_z_DEM_ISB_OUT0  ( )
  ,.ch2_z_DEM_ISB_OUT1  ( )
  ,.ch2_z_DEM_ISB_OUT2  ( )
  ,.ch2_z_DEM_ISB_OUT3  ( )
  ,.ch2_z_DEM_LSB_OUT0  ( )
  ,.ch2_z_DEM_LSB_OUT1  ( )
  ,.ch2_z_DEM_LSB_OUT2  ( )
  ,.ch2_z_DEM_LSB_OUT3  ( )
  ,.ch2_z_DEM_MSB_vld   ( )
  ,.ch2_dc_bias_o       ( )
  ,.ch2_dc_bias_latch   ( )
  `endif  
  //------------------------------Ch3 DSP data out----------------------------------------------------
  `ifdef CHANNEL_Z_ON
  ,.ch3_z_DEM_MSB_OUT0  ( ) 
  ,.ch3_z_DEM_MSB_OUT1  ( )
  ,.ch3_z_DEM_MSB_OUT2  ( )
  ,.ch3_z_DEM_MSB_OUT3  ( ) 
  ,.ch3_z_DEM_ISB_OUT0  ( )
  ,.ch3_z_DEM_ISB_OUT1  ( )
  ,.ch3_z_DEM_ISB_OUT2  ( )
  ,.ch3_z_DEM_ISB_OUT3  ( )
  ,.ch3_z_DEM_LSB_OUT0  ( )
  ,.ch3_z_DEM_LSB_OUT1  ( )
  ,.ch3_z_DEM_LSB_OUT2  ( )
  ,.ch3_z_DEM_LSB_OUT3  ( )
  ,.ch3_z_DEM_MSB_vld   ( )
  ,.ch3_dc_bias_o       ( )
  ,.ch3_dc_bias_latch   ( )
  `endif  
  `endif
  //+++++++++++++++++++++++++++++++++++++++++++++//
  //                  PIN END                   //                    
  //+++++++++++++++++++++++++++++++++++++++++++++//

 
);

thermo2binary_top Ch0_0_thermo2binary_top (
   .clk                                   ( clk           )
  ,.DEM_MSB_IN                            ( ch0_z_DEM_MSB_OUT0    )
  ,.DEM_ISB_IN                            ( ch0_z_DEM_ISB_OUT0    )
  ,.DEM_LSB_IN                            ( ch0_z_DEM_LSB_OUT0    )
  ,.DOUT                                  ( ch0_z_dsp_dout0       )
);
thermo2binary_top Ch0_1_thermo2binary_top (
   .clk                                   ( clk              )
  ,.DEM_MSB_IN                            ( ch0_z_DEM_MSB_OUT1    )
  ,.DEM_ISB_IN                            ( ch0_z_DEM_ISB_OUT1    )
  ,.DEM_LSB_IN                            ( ch0_z_DEM_LSB_OUT1    )
  ,.DOUT                                  ( ch0_z_dsp_dout1       )
);
thermo2binary_top Ch0_2_thermo2binary_top (
   .clk                                   ( clk              )
  ,.DEM_MSB_IN                            ( ch0_z_DEM_MSB_OUT2    )
  ,.DEM_ISB_IN                            ( ch0_z_DEM_ISB_OUT2    )
  ,.DEM_LSB_IN                            ( ch0_z_DEM_LSB_OUT2    )
  ,.DOUT                                  ( ch0_z_dsp_dout2       )
);
thermo2binary_top Ch0_3_thermo2binary_top (
   .clk                                   ( clk              )
  ,.DEM_MSB_IN                            ( ch0_z_DEM_MSB_OUT3    )
  ,.DEM_ISB_IN                            ( ch0_z_DEM_ISB_OUT3    )
  ,.DEM_LSB_IN                            ( ch0_z_DEM_LSB_OUT3    )
  ,.DOUT                                  ( ch0_z_dsp_dout3       )
);


endmodule
